@@ -70,7 +70,8 @@ enum{
 
 // Normalize an angle to +/-180degrees
 
-#define NORM_ANGLE( x )  while( x >180.0 ) x -= 360.0; while( x < -180.0 ) x += 360.0;
+#define NORM_ANGLE( x )      while( x >180.0 ) x -= 360.0; while( x < -180.0 ) x += 360.0;
+#define NORM_ANGLE_RAD( x )  while( (x) >PI ) (x) -= 2 * PI; while( (x) < -PI ) (x) += 2 * PI;
 
 // Convert degree to radian
 
@@ -175,6 +176,11 @@ typedef struct perspective_Prefs pPrefs;
 
 struct optVars{									//  Indicate to optimizer which variables to optimize
 		int hfov;								//  optimize hfov? 0-no 1-yes , etc
+
+  // panotools uses these variables for two purposes: to 
+  // determine which variables are used for reference to another one
+  // and to determine which variables to optimize
+
 		int yaw;				
 		int pitch;				
 		int roll;				
@@ -185,6 +191,20 @@ struct optVars{									//  Indicate to optimizer which variables to optimize
 		int e;
 		int shear_x;
 		int shear_y;
+		int tiltXopt;
+		int tiltYopt;
+		int tiltZopt;
+		int tiltScaleOpt;
+  
+		int transXopt;
+		int transYopt;
+		int transZopt;
+
+		int testP0opt;
+		int testP1opt;
+		int testP2opt;
+		int testP3opt;
+
 		};
 		
 typedef struct optVars optVars;
@@ -358,6 +378,15 @@ struct MakeParams{								// Actual parameters used by Xform functions for pano-
 	double  distance;
 	double	horizontal;
 	double	vertical;
+
+    // Tilt 
+	double tilt[4]; // 0 around x, 1 around y, 2 around z, 3 scaling factor
+    // Translation of cmaera plane
+    double trans[3];
+    // For testing new projections
+    double test[4];
+
+
 	Image *im;
 	Image *pn;
 	};
@@ -808,11 +837,14 @@ int execute_stack_new               ( double x_dest,double  y_dest, double* x_sr
 
 int resize				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );		
 int shear				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
+int shearInv				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int horiz				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int vert				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int radial				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int radial_brown			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 
+int tiltForward				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
+int tiltInverse				( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 
 int persp_sphere		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int persp_rect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
@@ -830,6 +862,8 @@ int sphere_tp_rect		( double x_dest,double  y_dest, double* x_src, double* y_src
 int sphere_cp_pano		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int rect_erect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int erect_rect			( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
+int plane_transfer_to_camera( double x_dest, double y_dest, double * x_src, double * y_src, void * params);
+int plane_transfer_from_camera( double x_dest, double y_dest, double * x_src, double * y_src, void * params);
 int erect_sphere_tp	( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );	
 int mirror_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int mercator_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
@@ -854,6 +888,11 @@ int erect_panini		( double x_dest,double  y_dest, double* x_src, double* y_src,
 int equipanini_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_equipanini		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 
+int panini_general_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+int erect_panini_general		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
+Image * setup_panini_general	( struct MakeParams * pmp );
+int maxFOVs_panini_general	( double *params, double *fovs );
+
 int arch_erect		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int erect_arch		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 
@@ -870,8 +909,8 @@ int sphere_cp_mirror	( double x_dest,double  y_dest, double* x_src, double* y_sr
 
 int sphere_tp_pano		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
 int pano_sphere_tp		( double x_dest,double  y_dest, double* x_src, double* y_src, void* params );
-//int sphere_tp_mirror( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
-//int mirror_sphere_tp( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
+int sphere_tp_mirror( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
+int mirror_sphere_tp( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
 int sphere_tp_equisolid( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
 int equisolid_sphere_tp( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
 int sphere_tp_orthographic( double x_dest,double  y_dest, double* x_src, double* y_src, void* params);
